export default class ManagedData {
    constructor(data) {
        this.data = data;
        this.history = []; // Newest state is always at 0
        this.historyPosition = 0;
        this.unsavedChanges = false;

        this.storeCurrentData("Initial state", false);
    }

    updateUnsavedChangesHandler() {
        if (this.hasUnsavedChanges()) {
            window.addEventListener("beforeunload", this.handleBeforeUnload);
        } else {
            window.removeEventListener("beforeunload", this.handleBeforeUnload);
        }
    }

    handleBeforeUnload(e) {
        var confirmationMessage = 'If you leave before saving, unsaved changes will be lost.';
    
        (e || window.event).returnValue = confirmationMessage; //Gecko + IE
        return confirmationMessage; //Gecko + Webkit, Safari, Chrome etc.
    }

    hasUnsavedChanges() {
        return this.unsavedChanges;
    }

    saveChanges() {
        this.unsavedChanges = false;
        this.updateUnsavedChangesHandler();
    }

    onUndo() {}
    onRedo() {}

    undo() {
        if (this.step(1)) {
            this.onUndo();
            return true;
        } else {
            return false;
        }
    }

    redo() {
        if (this.step(-1)) {
            this.onRedo();
            return true;
        } else {
            return false;
        }
    }

    step(direction = 1) {
        var newHistoryPosition = this.historyPosition + Math.sign(direction);

        if (
            newHistoryPosition >= this.history.length ||
            newHistoryPosition < 0
        ) {
            return false;
        }

        this.historyPosition = newHistoryPosition;
        this.data = JSON.parse(this.history[this.historyPosition].data);

        return true;
    }

    storableData(data) {
        return data;
    }

    storeCurrentData(description, relevantChanges = true) {
        var formattedData = this.storableData(this.data);

        this.history.unshift({
            description: description,
            data: JSON.stringify(formattedData), // Creating a deep copy
        });

        // Forget about the currently stored potential future
        this.history.splice(0, this.historyPosition);
        this.historyPosition = 0;

        if (!this.hasUnsavedChanges() && relevantChanges) {
            this.unsavedChanges = true;
            this.updateUnsavedChangesHandler();
        }
    }
}
